package simplerace;

import wox.serial.Easy;

/**
 * Created by IntelliJ IDEA.
 * User: Pete
 * Date: 16-Feb-2007
 * Time: 00:39:27
 * To change this template use File | Settings | File Templates.
 */
public class ModularRMLPController implements Controller, Evolvable {

    SimpleRMLPController solo;
    MLP decision;

    public ModularRMLPController () {
        this.solo = (SimpleRMLPController) Easy.load("SimpleRMLPEvolved.xml");
        this.decision = new MLP (3, 2, 1);
    }

    private ModularRMLPController (SimpleRMLPController sol, MLP dec) {
        this.solo = sol;
        this.decision = dec;
    }

    public int control (SensorModel model) {
        double speed = model.getSpeed();
        double angle = model.getAngleToNextWaypoint();
        double dist = model.getDistanceToNextWaypoint();
        if ( model.otherVehicleIsPresent() ) {
            double otherSpeed = getOtherSpeed( model.getOtherVehicleVelocity(),
                                               model.getOtherVehicleOrientation() );
            double otherDist = getOtherDistanceToNextWaypoint( model.getNextWaypointPosition(),
                                                                model.getOtherVehiclePosition() );
            double[] inputs = { 1, dist / otherDist, speed / otherSpeed };
            double[] output = decision.propagate (inputs);
            double decRes = output[0];
            if ( decRes < 0.0 ) {
                angle = model.getAngleToNextNextWaypoint();
                dist = model.getDistanceToNextNextWaypoint();
            }
        }
        return solo.control( speed, angle, dist );
    }

    public void mutate () {
        solo.mutate ();
        decision.mutate ();
    }

    public Evolvable copy () {
        return new ModularRMLPController ( solo.copyExplicit (), decision.copy() );
    }

    public void reset () {
        solo.reset ();
        decision.reset ();
    }

    public double getOtherSpeed ( Vector2d velocity, double orientation ) {
            double velocityMagnitude = velocity.mag ();
            double directionOfMovement = Math.atan2 (velocity.y, velocity.x);
            double directionDifference = directionOfMovement - orientation;
            return Math.cos (directionDifference) * velocityMagnitude;
    }

    public double getOtherDistanceToNextWaypoint( Vector2d waypoint, Vector2d pos ) {
        int width = 400;
        int height = 400;
        int diagonal = (int) Math.sqrt (width * width + height * height);
        return pos.dist(waypoint) / diagonal;
   }
}
